(**************************************************************************)
(*                                                                        *)
(*                                 OCaml                                  *)
(*                                                                        *)
(*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           *)
(*                                                                        *)
(*   Copyright 1996 Institut National de Recherche en Informatique et     *)
(*     en Automatique.                                                    *)
(*                                                                        *)
(*   All rights reserved.  This file is distributed under the terms of    *)
(*   the GNU Lesser General Public License version 2.1, with the          *)
(*   special exception on linking described in the file LICENSE.          *)
(*                                                                        *)
(**************************************************************************)

open Clambda
open Format

val clambda: formatter -> ulambda -> unit
val approx: formatter -> value_approximation -> unit
val structured_constant: formatter -> ustructured_constant -> unit

val phantom_defining_expr_opt
   : formatter
  -> uphantom_defining_expr option
  -> unit
