val opACC0 : int
val opACC1 : int
val opACC2 : int
val opACC3 : int
val opACC4 : int
val opACC5 : int
val opACC6 : int
val opACC7 : int
val opACC : int
val opPUSH : int
val opPUSHACC0 : int
val opPUSHACC1 : int
val opPUSHACC2 : int
val opPUSHACC3 : int
val opPUSHACC4 : int
val opPUSHACC5 : int
val opPUSHACC6 : int
val opPUSHACC7 : int
val opPUSHACC : int
val opPOP : int
val opASSIGN : int
val opENVACC1 : int
val opENVACC2 : int
val opENVACC3 : int
val opENVACC4 : int
val opENVACC : int
val opPUSHENVACC1 : int
val opPUSHENVACC2 : int
val opPUSHENVACC3 : int
val opPUSHENVACC4 : int
val opPUSHENVACC : int
val opPUSH_RETADDR : int
val opAPPLY : int
val opAPPLY1 : int
val opAPPLY2 : int
val opAPPLY3 : int
val opAPPTERM : int
val opAPPTERM1 : int
val opAPPTERM2 : int
val opAPPTERM3 : int
val opRETURN : int
val opRESTART : int
val opGRAB : int
val opCLOSURE : int
val opCLOSUREREC : int
val opOFFSETCLOSUREM3 : int
val opOFFSETCLOSURE0 : int
val opOFFSETCLOSURE3 : int
val opOFFSETCLOSURE : int
val opPUSHOFFSETCLOSUREM3 : int
val opPUSHOFFSETCLOSURE0 : int
val opPUSHOFFSETCLOSURE3 : int
val opPUSHOFFSETCLOSURE : int
val opGETGLOBAL : int
val opPUSHGETGLOBAL : int
val opGETGLOBALFIELD : int
val opPUSHGETGLOBALFIELD : int
val opSETGLOBAL : int
val opATOM0 : int
val opATOM : int
val opPUSHATOM0 : int
val opPUSHATOM : int
val opMAKEBLOCK : int
val opMAKEBLOCK1 : int
val opMAKEBLOCK2 : int
val opMAKEBLOCK3 : int
val opMAKEFLOATBLOCK : int
val opGETFIELD0 : int
val opGETFIELD1 : int
val opGETFIELD2 : int
val opGETFIELD3 : int
val opGETFIELD : int
val opGETFLOATFIELD : int
val opSETFIELD0 : int
val opSETFIELD1 : int
val opSETFIELD2 : int
val opSETFIELD3 : int
val opSETFIELD : int
val opSETFLOATFIELD : int
val opVECTLENGTH : int
val opGETVECTITEM : int
val opSETVECTITEM : int
val opGETBYTESCHAR : int
val opSETBYTESCHAR : int
val opBRANCH : int
val opBRANCHIF : int
val opBRANCHIFNOT : int
val opSWITCH : int
val opBOOLNOT : int
val opPUSHTRAP : int
val opPOPTRAP : int
val opRAISE : int
val opCHECK_SIGNALS : int
val opC_CALL1 : int
val opC_CALL2 : int
val opC_CALL3 : int
val opC_CALL4 : int
val opC_CALL5 : int
val opC_CALLN : int
val opCONST0 : int
val opCONST1 : int
val opCONST2 : int
val opCONST3 : int
val opCONSTINT : int
val opPUSHCONST0 : int
val opPUSHCONST1 : int
val opPUSHCONST2 : int
val opPUSHCONST3 : int
val opPUSHCONSTINT : int
val opNEGINT : int
val opADDINT : int
val opSUBINT : int
val opMULINT : int
val opDIVINT : int
val opMODINT : int
val opANDINT : int
val opORINT : int
val opXORINT : int
val opLSLINT : int
val opLSRINT : int
val opASRINT : int
val opEQ : int
val opNEQ : int
val opLTINT : int
val opLEINT : int
val opGTINT : int
val opGEINT : int
val opOFFSETINT : int
val opOFFSETREF : int
val opISINT : int
val opGETMETHOD : int
val opBEQ : int
val opBNEQ : int
val opBLTINT : int
val opBLEINT : int
val opBGTINT : int
val opBGEINT : int
val opULTINT : int
val opUGEINT : int
val opBULTINT : int
val opBUGEINT : int
val opGETPUBMET : int
val opGETDYNMET : int
val opSTOP : int
val opEVENT : int
val opBREAK : int
val opRERAISE : int
val opRAISE_NOTRACE : int
val opGETSTRINGCHAR : int
val opPERFORM : int
val opRESUME : int
val opRESUMETERM : int
val opREPERFORMTERM : int
